/* 
 * File:   timebase.c
 * Author: WHalphen
 *
 * Created on January 4, 2016
 */

/* 
 * This file must be built without optimization or the delay loops will be
 * killed by the optimizer.  To prevent optimization of the file, right click
 * on the file and select 'Override build options'.  Then turn off the
 * optimization.
 */

#include <p24FV32KA301.h>

#include "timebase.h"

volatile static unsigned long tb_counter = 0; //tenths of milliseconds counter -- updated by timer interrupt
volatile static int tenths = 10; // 10 counts to get to milliseconds

// start timer

void tb_start(void) {
    T2CONbits.TON = 1;
}

// stop timer

void tb_stop(void) {
    T2CONbits.TON = 0;
}

struct pwm_timer {
    int on_time;
    int off_time;
    int dig_out;
    int invert;
    int state;
    int counter;
};

volatile static struct pwm_timer pwm_timers[NUM_TIMERS];
volatile static unsigned int triggered = 0; // set to 1 to indicated trigger input
volatile static unsigned int state = 0; // trigger debounce status

void __attribute__((__interrupt__, auto_psv)) _T2Interrupt(void) {
    int i;
    tb_counter++;
    if (!tenths--) { // check if time to manage the timers
        tenths = 9; // 10 counts, minus this pass thru the isr

        // debounce the trigger switch
        state = (state << 1) | TRIGGER | 0xe000;
        if (state == 0xf000) triggered = 1;
        
        for (i = 0; i < NUM_TIMERS; i++) { // service the pwm timers
            if (pwm_timers[i].counter != 0) pwm_timers[i].counter--; // decrement timer
            else { // ctr is at 0, service the timer
                if (pwm_timers[i].on_time == 0) {
                    pwm_timers[i].state = 0;
                } else if (pwm_timers[i].off_time == 0) { // one-shot
                    pwm_timers[i].state = 1;
                    pwm_timers[i].counter = pwm_timers[i].on_time;
                    pwm_timers[i].on_time = 0;
                } else { // pwm
                    if (pwm_timers[i].state == 0) { // toggle to ON
                        pwm_timers[i].state = 1;
                        pwm_timers[i].counter = pwm_timers[i].on_time;
                    } else { // toggle to OFF
                        pwm_timers[i].state = 0;
                        pwm_timers[i].counter = pwm_timers[i].off_time;
                    }
                }
            }
            switch (pwm_timers[i].dig_out) {
                case 0:
                    break;
                case 1:
                    DOUT1 = pwm_timers[i].state ^ pwm_timers[i].invert;
                    break;
                case 2:
                    DOUT2 = pwm_timers[i].state ^ pwm_timers[i].invert;
                    break;
                case 3:
                    DOUT3 = pwm_timers[i].state ^ pwm_timers[i].invert;
                    break;
                case 4:
                    DOUT4 = pwm_timers[i].state ^ pwm_timers[i].invert;
                    break;
                case 5:
                    DOUT5 = pwm_timers[i].state ^ pwm_timers[i].invert;
            }
        }
    }
    IFS0bits.T2IF = 0; //Clear the Timer2 interrupt status flag       
}

void init_pwm_timers(void) {
    int i;
    for(i=0; i<NUM_TIMERS; i++){
        pwm_timers[i].on_time = 0;
        pwm_timers[i].off_time = 0;
        pwm_timers[i].state = 0;
        pwm_timers[i].dig_out = 0;
        pwm_timers[i].invert = 0;
        pwm_timers[i].counter = 0;
    }
}

// initialize and start the timebase

void tb_initialize(unsigned int count) {
    //instr clk src; no prescale
    T2CON = 0; // stop timer and reset control register
    TMR2 = 0; // zero contents of timer
    PR2 = count; // load period register
    init_pwm_timers();
    IFS0bits.T2IF = 0; //Clear the Timer2 interrupt status flag
    IEC0bits.T2IE = 1; //Enable Timer2 interrupts
    T2CONbits.TON = 1; // Start timer
}

// get counts since timer initialized

unsigned long tb_count() {
    return tb_counter;
}

void tb_delay_ms(unsigned int milliseconds) {
    unsigned long end;
    unsigned long ctr;
    end = tb_counter + (unsigned long) milliseconds * 10;
    ctr = tb_counter;
    while (ctr < end) ctr = tb_counter;
}

void tb_delay_100us(unsigned int count) {
    unsigned long end;
    unsigned long ctr;
    end = tb_counter + (unsigned long) count;
    ctr = tb_counter;
    while (ctr < end) ctr = tb_counter;
}

void tb_clear_counter() {
    tb_counter = 0;
}

void pwm_timer_set(int tmr_num, int on, int off, int dig_out, int invert) {
    pwm_timers[tmr_num].on_time = on;
    pwm_timers[tmr_num].off_time = off;
    pwm_timers[tmr_num].dig_out = dig_out;
    pwm_timers[tmr_num].invert = invert;
}

void pwm_timer_reset(int n) {
    pwm_timers[n].on_time = 0;
    pwm_timers[n].off_time = 0;
    pwm_timers[n].counter = 0;
    pwm_timers[n].state = 0;
    pwm_timers[n].invert = 0;
    switch (pwm_timers[n].dig_out) {
        case 0:
            break;
        case 1:
            DOUT1 = 0;
            break;
        case 2:
            DOUT2 = 0;
            break;
        case 3:
            DOUT3 = 0;
            break;
        case 4:
            DOUT4 = 0;
            break;
        case 5:
            DOUT5 = 0;
    }
}

int pwm_timer(int n){
    return pwm_timers[n].state;
}

int trigger_status(void) {
    return triggered;
}

void trigger_clear(void) {
    triggered = 0;
}
