/* 
 * File:   timebase.h
 * Author: WHalphen
 *
 * Created on October 20,2016
 */

#ifndef TIMEBASE_H
#define	TIMEBASE_H

#include "sys_config.h"
#include <stdio.h>

/* Routines for a timebase for the PIC24FxxK30x processor
 * 
 * One timer is used to count system clock ticks.  Every 0.1 millisecond, the
 * counter is reloaded and an interrupt is used to update a millisecond counter.
 * 
 * Uses Timer 2 and interrupt.
 * 
 * Use tb_initialize(count) to create and start the timebase.  'count' is the
 * number of instruction cycles per 0.01 millisecond for the current processor
 * configuration.   (Fosc = 32 MHz, resulting in instruction clock of 16 MHz.)
 * The tb_count() property returns the number of tenth millisecond counts
 * since the initialization of the timebase, or since the tb_clear_counter()
 * function was last executed.  The tb_count() property value maxes out and
 * rolls over to zero after 2^32 counts.
 * 
 * The following functions are available:
 * tb_start() -- sets up and starts the timer, zeroes the counter
 * tb_stop() -- stops the timer
 * tb_time_ms() -- returns the number of milliseconds since timer initialized
 *   (valid for 2^32/10 milliseconds (~4.97 days) from initialization)
 * tb_delay_ms(milliseconds) -- delays for specified number of milliseconds
 * tb_delay_100us(count) -- delays for specified count of tenths of milliseconds
 * tb_clear_counter() -- clears tenths of milliseconds_counter
 * tb_initialize(count) -- initializes and starts timebase, and counter
 * tb_count() -- returns tenths of milliseconds since timer initiallized
 * void pwm_timer_set(int tmr_num, int on, int off, int dig_out, int invert);
 *    sets specified timer for one-shot or pwm output
 *    parameters:
 *       tmr_num -- specified timer to be used
 *       on -- on time in milliseconds
 *       off -- in the case of PWM, off time in milliseconds - if 0, one-shot timer
 *       dig_out -- selects digital output to be driven by the timer - if 0, no dig output
 *       invert -- in the case of digital output, 1 will cause output to be inverted
 * void pwm_timer_reset(int n); -- immediately stops and resets the specified timer
 * int pwm_timer(int n); -- returns the state of the specified timer
 * void init_pwm_timers(void); -- initializes the timers -- execute once during startup
 */

// start timer
void tb_start(void);

// stop timer
void tb_stop(void);  

// initialize and start the timebase
void tb_initialize(unsigned int count);

// get counts since timer initialized
unsigned long tb_count();

void tb_delay_ms(unsigned int milliseconds);     

void tb_delay_100us(unsigned int count);     

void tb_clear_counter();

void __attribute__((__interrupt__)) _T2Interrupt(void);

void pwm_timer_set(int tmr_num, int on, int off, int dig_out, int invert);
void pwm_timer_reset(int n);
int pwm_timer(int n); // returns state of timer n
void init_pwm_timers(void);

int trigger_status(void);
void trigger_clear(void);

#endif	/* TIMEBASE_H */

